# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 最新版本的用户装饰器
@created_time: 2023/5/29 10:02
"""
from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from django.http import JsonResponse
from rest_framework.request import Request

from .custom_tool import dynamic_load_class


def parse_header(token: str = None):
    """解析用户token验证"""
    UserService, import_err = dynamic_load_class(import_path="xj_user.services.user_service", class_name="UserService")
    if import_err:
        return {}, "该系统被没有安装用户模块"
    return UserService.check_token(token)


def user_authentication_wrapper(func):
    """
    用户认证装饰器，如果有Authorization则检查用户的有效性，如没有则认为用户以游客访问
    """

    def wrapper(instance, arg_request=None, *args, request=None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param request APIView实例会传入请求包
        @param request APIView实例会传入请求包
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        # =========== section 解析系统request对象 start ==================
        # print("> user_authentication_wrapper:", instance, request, args, kwargs)
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request
        if request is None:
            return func(instance, request, *args, request=request, request_params={}, **kwargs, )
        # =========== section 解析系统request对象 end   ==================

        # =========== section 获取token并验证 start ==================
        token = request.headers.get('Authorization', None)
        # 如果没有传token则可视为以游客身份访问
        if not token or str(token) == 'null' or str(token).strip().upper() == "BEARER":
            return func(instance, request=request, user_info={}, *args, **kwargs)
        # 动态加载用户模块，进行权限验证
        UserService, import_err = dynamic_load_class(import_path="xj_user.services.user_service", class_name="UserService")
        if import_err:
            return JsonResponse({'err': 6001, 'msg': "该系统被没有安装用户模块"})
        user_serv, error_text = UserService.check_token(token)
        if error_text:
            return JsonResponse({'err': 6001, 'msg': error_text})
        # =========== section 获取token并验证 start ==================

        # =========== section 解析用户信息并返回 start ==================
        request.user = user_serv
        result = func(instance, *args, request=request, user_info=user_serv, **kwargs)
        return result
        # =========== section 解析用户信息并返回 end   ==================

    return wrapper


def user_authentication_force_wrapper(func):
    """
    用户认证装饰器，如果有Authorization则检查用户的有效性，如没有则认为用户以游客访问
    """

    def wrapper(*args, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param request APIView实例会传入请求包
        @param request APIView实例会传入请求包
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        """
        # =========== section 解析系统request对象 start ==================
        # print("> user_authentication_wrapper:", instance, request, args, kwargs)
        request = None
        for ins in args:
            if isinstance(ins, WSGIRequest) or isinstance(ins, Request) or isinstance(ins, ASGIRequest):
                request = ins
        for ins_name, ins in kwargs.items():
            if isinstance(ins, WSGIRequest) or isinstance(ins, Request) or isinstance(ins, ASGIRequest):
                request = ins

        if request is None:
            return func(*args, **kwargs)
        # =========== section 解析系统request对象 end   ==================

        # =========== section 获取token并验证 start ==================
        token = request.headers.get('Authorization', None)
        # 没有token直接返回6000异常
        if not token or str(token) == 'null' or str(token).strip().upper() == "BEARER":
            return JsonResponse({'err': 6000, 'msg': '该用户未登录'})
        # 动态加载用户模块，进行权限验证
        UserService, import_err = dynamic_load_class(import_path="xj_user.services.user_service", class_name="UserService")
        if import_err:
            return JsonResponse({'err': 6001, 'msg': "该系统被没有安装用户模块"})
        user_serv, error_text = UserService.check_token(token)
        if error_text:
            return JsonResponse({'err': 6001, 'msg': error_text})
        # =========== section 获取token并验证 start ==================

        # =========== section 解析用户信息并返回 start ==================
        request.user = user_serv
        kwargs.pop("user_info", None)
        result = func(*args, user_info=user_serv, **kwargs)
        return result
        # =========== section 解析用户信息并返回 end   ==================

    return wrapper



# encoding: utf-8
"""
@project: djangoModel->module_service_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 模块开放服务注册
@created_time: 2023/6/5 9:28
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.xj_module_service_services import XJModuleServiceServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class ModuleApiView(APIView):

    @api_view(["GET", "POST"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def services_list(self, *args, request_params, **kwargs):
        data, err = XJModuleServiceServices.services_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)




# encoding: utf-8
"""
@project: djangoModel->get_system_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2023/6/14 15:43
"""
from importlib import import_module
import re

from django.contrib.admindocs.views import simplify_regex
from django.urls import URLResolver, URLPattern
from rest_framework.views import APIView
import six

from main import settings


class GetSystemApis():
    """
    获取本项目的API列表
    """

    _PATH_PARAMETER_COMPONENT_RE = re.compile(r'<(?:(?P<converter>[^>:]+):)?(?P<parameter>\w+)>')

    def get_apis(self):
        data = []

        urlconf = settings.ROOT_URLCONF

        if not isinstance(urlconf, six.string_types):
            return []

        urls = import_module(urlconf)

        patterns = urls.urlpatterns

        data = self.__get_api_endpoints(patterns)
        return data

    def __is_api_view(self, callback):
        """
        如果给定的视图回调是DRF View/viewset，则返回True
        """
        cls = getattr(callback, 'cls', None)
        return (cls is not None) and issubclass(cls, APIView)

    def endpoint_ordering(self, endpoint):
        """
        返回排序字段
        :param endpoint:
        :return:
        """
        # todo: 新增返回字段信息
        path, method, name, describe = endpoint.values()
        method_priority = {
            'GET': 0,
            'POST': 1,
            'PUT': 2,
            'PATCH': 3,
            'DELETE': 4
        }.get(method, 5)
        return (path, method_priority)

    def __get_path_from_regex(self, path_regex):
        """
        给定URL conf正则表达式，返回URI模板字符串
        """
        path = simplify_regex(path_regex)
        path = re.sub(self._PATH_PARAMETER_COMPONENT_RE, r'{\g<parameter>}', path)
        return path

    def __should_include_endpoint(self, path, callback):
        """
        如果应该包含给定的endpoint，则返回“ True”
        """
        if not self.__is_api_view(callback):
            return False

        if callback.cls.schema is None:
            return False

        if 'schema' in callback.initkwargs:
            if callback.initkwargs['schema'] is None:
                return False

        if path.endswith('.{format}') or path.endswith('.{format}/'):
            return False

        return True

    def __get_allowed_methods(self, callback):
        """
        返回允许的http method列表
        """
        if hasattr(callback, 'actions'):
            actions = set(callback.actions)
            http_method_names = set(callback.cls.http_method_names)
            methods = [method.upper() for method in actions & http_method_names]
        else:
            methods = callback.cls().allowed_methods

        return [method for method in methods if method not in ('OPTIONS', 'HEAD')]

    def __get_api_endpoints(self, patterns=None, prefix=''):
        """
        通过检查URL conf返回所有可用API端点的列表
        """
        if patterns is None:
            patterns = patterns

        api_endpoints = []

        for pattern in patterns:
            # path_regex = prefix + get_original_route(pattern)
            path_regex = prefix + pattern.pattern.regex.pattern.replace('\\', '')
            if isinstance(pattern, URLPattern):
                path = self.__get_path_from_regex(path_regex)
                callback = pattern.callback
                if self.__should_include_endpoint(path, callback):
                    for method in self.__get_allowed_methods(callback):
                        endpoint = {"api": path,
                                    "describe": str(callback.initkwargs['description']).strip().split('\n')[0] if
                                    callback.initkwargs.get('description') else
                                    str(callback.__doc__).strip().split('\n')[0],
                                    # todo: 从 name 中获取备注信息
                                    "name": pattern.name,
                                    "method": method}
                        api_endpoints.append(endpoint)

            elif isinstance(pattern, URLResolver):
                nested_endpoints = self.__get_api_endpoints(
                    patterns=pattern.url_patterns,
                    prefix=path_regex
                )
                api_endpoints.extend(nested_endpoints)

        api_endpoints = sorted(api_endpoints, key=self.endpoint_ordering)

        return api_endpoints



# encoding: utf-8
"""
@project: djangoModel->translate_server
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 翻译公共服务
@created_time: 2022/11/17 11:20
"""
import hashlib
import json
import random
import re
import time

import requests

appKey = "z7EtfGBkVNN2OnQC49vd4ns1"
app_sercet = "CIvWKy78t6T9d56vORQvItx9tHLCsLqK"


def replace():
    pass


class TranslateService():
    old_text_arr = []
    replace_symbol = {}

    def get_sign(self, form_data):
        # 第一步
        sorted_form_data = {}
        for i in sorted(form_data):
            sorted_form_data[i] = form_data[i]

        # 第二步
        old_txt = sorted_form_data["text"]
        text = ""
        for t in old_txt:
            text += t
        sorted_form_data["text"] = text

        # 第三步
        urlencode_str = ""
        for k, v in sorted_form_data.items():
            if not urlencode_str == "":
                urlencode_str += "&"
            urlencode_str += k + "=" + v

        # 第四步
        md5_str = urlencode_str + "&" + app_sercet

        # 第五步
        m1 = hashlib.md5()
        m1.update(md5_str.encode("utf-8"))
        sign = m1.hexdigest()

        sorted_form_data["sign"] = sign
        sorted_form_data["text"] = old_txt
        return sorted_form_data

    def translate(self, text_arr, source="zh", target="en"):
        # 完成sign 数据拼接
        form_data = {
            "appKey": "z7EtfGBkVNN2OnQC49vd4ns1",
            'source': source,
            'target': target,
            'text': text_arr
        }
        form_data = self.get_sign(form_data)
        # 发送请求
        session = requests.session()
        header = {
            "Content-Type": "application/json",
            "User-Agent": "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.100 Safari/537.36"
        }
        return session.post(url="https://dev-web.tangpafanyi.com/transaction/api/text/tran", headers=header, data=json.dumps(form_data)).json(), None

    def article_cut_lines(self, article):
        return article.split("\n")

    def markdown_paragraph_pick(self, text):
        # 段落文本解析
        # 满足下面条件则拿出初步翻译的文本进行替换
        find_patt = ""
        translate_patt_list = [
            "(^#{1,6}\s)([^\n]*)?",  # 标题
            "(^\>\s)([^\n]*)?",  # 引用
            "(^\s*\*\s\[x?\])([^\n]*)?",  # 任务列表
            "(^\s*\d*\.\s)([^\n]*)?",  # 有序列表
            "(^\s*\*\s)([^\n^]*)?",  # 无序列表
            "(^\s*\-\s)([^\n^]*)?",  # 脑图
        ]
        for patt in translate_patt_list:
            res = re.match(patt, text)
            if not res:
                continue
            find_patt, res_str = res.groups()
            res_str = self.__replace_symbol(res_str)
            return find_patt, res_str

        return find_patt, self.__replace_symbol(text)

    def __replace_symbol(self, text):
        # 替换符号
        replace = re.findall("\([^\n^\)]*?\)", text)
        for i in replace:
            name = self.random_name()
            self.replace_symbol[name] = i
            text = text.replace(i, name)
        return text

    def deacidizing_symbol(self, text):
        # 翻译完成官员符号
        for k, v in self.replace_symbol.items():
            text = text.replace(k, v)
        return text

    # 随机替换字符串
    def random_name(self):
        return "{{" + str(random.randint(9999, 100000)) + str(int(round(time.time() * 1000)))[-3:-1] + "}}"

#
# if __name__ == '__main__':
#     app = TranslateService()
#     article = """
#     [Vditor - 浏览器端的 Markdown 编辑器](https://ld246.com/article/1549638745630) 是一款**所见即所得**编辑器，支持 *Markdown*。
#
#     * 不熟悉 Markdown 可使用工具栏或~~快捷键~~进行排版
#       * 熟悉 Markdown 可直接**排版**，也*可切换*为分屏预~~览打~~算
#
#     更多细节和用法请参考 [Vditor - 浏览器端的 Markdown 编辑器](https://ld246.com/article/1549638745630)，同时也欢迎向我们提出建议或报告问题，谢谢 ❤️
#
#     ## 教程
#
#     这是一篇讲解如何正确使用 **Markdown** 的排版示例，学会这个很有必要，能让你的文章有更佳清晰的排版。
#
#     > 引用文本：Markdown is a text formatting syntax inspired
#     """
#
#     old_line_arr = app.article_cut_lines(article)
#
#     markdown_symbol_list = []
#     translate_list = []
#
#     for line in old_line_arr:
#         markdown_symbol, translate_text = app.markdown_paragraph_pick(line)
#         markdown_symbol_list.append(markdown_symbol)
#         translate_list.append(translate_text)
#
#     result, err = app.translate(translate_list)
#     translated_list = [i["text"] for i in result["data"][0]["translated"]]
#     deacidizing_list = []
#     for i, j in zip(translated_list, markdown_symbol_list):
#         deacidizing_list.append(j + i)
#
#     deacidizing = ""
#     for i in deacidizing_list:
#         deacidizing += app.deacidizing_symbol(i) + "\n"
#     print(deacidizing)



# encoding: utf-8
"""
@project: djangoModel->region_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/12/6 10:56
"""

from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from ..services.regin_service import RegionService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data
from ..validate import RegionValidate


# =============== 地区操作 =======================
class RegionAPIS(APIView):
    # 查询列表
    @require_http_methods(["GET"])
    def get_region_tree(self):
        params = parse_data(self)
        p_code = params.get("code", None)
        service = RegionService()
        region_tree, err = service.region_tree(p_code)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=region_tree)

    # 查询列表
    @require_http_methods(["GET"])
    def region_list(self):
        params = parse_data(self)
        print(params)
        data, err = RegionService.list(params=params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(["POST"])
    def region_add(self, ):
        data = parse_data(self)
        validator = RegionValidate(data)
        is_valid, error = validator.validate()
        if not is_valid:
            return util_response(err=1000, msg=error)

        res, err = RegionService.add(data)

        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=res)

    @require_http_methods(["PUT"])
    def region_edit(self):
        data = parse_data(self)
        res, err = RegionService.edit(data)
        if err:
            return util_response(err=1000, msg=err)
        return util_response()

    @require_http_methods(["DELETE"])
    def region_del(self):
        params = parse_data(self)
        pk = params.get("id", None) or params.get("pk", None)
        data, err = RegionService.delete(pk)
        if not err is None:
            return util_response(err=1000, msg=err)
        return util_response()


class RegionCodeParse(APIView):

    def parse_code(self, code):
        # 解析code
        service = RegionService()
        res = service.cn_parse_location_code(code)
        return util_response()



# encoding: utf-8
"""
@project: djangoModel->notice_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 统计API
@created_time: 2023/3/1 15:12
"""
from rest_framework.views import APIView

from ..services.notice_services import NoticeServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class NoticeApis(APIView):

    @request_params_wrapper
    def notice_list(self, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}
        try:
            need_pagination = bool(request_params.pop("need_pagination", True))
        except Exception as e:
            need_pagination = False
        data, err = NoticeServices.notice_list(params=request_params, need_pagination=need_pagination)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @request_params_wrapper
    def type_list(self, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}
        try:
            need_pagination = int(request_params.pop("need_pagination", 0))
        except Exception as e:
            need_pagination = 0

        data, err = NoticeServices.notice_type_list(params=request_params, need_pagination=need_pagination)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def notice_add(self, *args, request_params=None, user_info=None, **kwargs):
        if user_info is None:
            user_info = {}
        if request_params is None:
            request_params = {}
        request_params.setdefault("from_user_id", user_info.get("user_id"))
        data, err = NoticeServices.notice_add(notice_params=request_params)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @request_params_wrapper
    def notice_edit(self, *args, request_params=None, **kwargs):
        if request_params is None:
            request_params = {}
        notice_id = request_params.pop("notice_id", None) or request_params.pop("id", None)
        data, err = NoticeServices.notice_edit(params=request_params, notice_id=notice_id)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->notice_services
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 消息提醒服务
@created_time: 2023/3/1 11:08
"""

from django.core.paginator import Paginator, EmptyPage
from django.db.models import F

from ..models import NoticeType, Notice
from ..utils.custom_tool import format_params_handle


class NoticeServices():
    """消息提醒服务类"""

    @staticmethod
    def notice_type_list(params: dict = None, need_pagination: int = 0):
        if params is None:
            params = {}
        page = params.pop("page", 1)
        size = params.pop("size", 10)
        # 参数过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "value", "name", "notice_type_id"],
            alias_dict={"notice_type_id": "id"}
        )

        notice_type_obj = NoticeType.objects.filter(**params).values("id", "value", "name", "description")
        # 不许分页直接返回
        if not need_pagination:
            notice_type_list = list(notice_type_obj)
            return notice_type_list, None

        total = notice_type_obj.count()
        paginator = Paginator(notice_type_obj, size)
        try:
            notice_type_obj = paginator.page(page)
        except EmptyPage:
            notice_type_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'

        return {"page": int(page), "size": int(size), "total": total, "list": list(notice_type_obj.object_list)}, None

    @staticmethod
    def notice_list(params: dict = None, need_pagination: bool = True):
        if params is None:
            params = {}
        page = params.pop("page", 1)
        size = params.pop("size", 10)
        # 参数过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id", "id_list", "notice_id_list", "notice_type_value", "notice_type_id", "notice_type_id_list", "thread_id", "thread_id_list",
                "from_user_id", "from_user_id_list", "to_user_id", "to_user_id_list", "to_role_id", "to_role_id_list", "status",
                "created_time_start", "created_time_end" "read_time_start", "read_time_end", "expire_time_start", "expire_time_end"
            ],
            alias_dict={
                "notice_id_list": "id__in",
                "id_list": "id__in",
                "notice_type_id_list": "notice_type_id__in",
                "thread_id_list": "thread_id__in",
                "from_user_id_list": "from_user_id__in",
                "to_user_id_list": "to_user_id__in",
                "to_role_id_list": "to_role_id__in",
                "created_time_start": "created_time__gte",
                "created_time_end": "created_time__lte",
                "read_time_start": "read_time__get",
                "read_time_end": "read_time__lte",
                "expire_time_start": "expire_time__gte",
                "expire_time_end": "expire_time__lte",
            }
        )

        notice_obj = Notice.objects.filter(**params).annotate(
            notice_type_name=F("notice_type__name"),
            notice_type_value=F("notice_type__value"),
        ).values()
        # 不许分页直接返回
        if not need_pagination:
            if not params:
                return None, "无法搜索，参数错误"
            notice_obj_list = list(notice_obj)
            return notice_obj_list, None

        total = notice_obj.count()
        paginator = Paginator(notice_obj, size)
        try:
            notice_obj = paginator.page(page)
        except EmptyPage:
            notice_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'

        return {"page": int(page), "size": int(size), "total": total, "list": list(notice_obj.object_list)}, None

    @staticmethod
    def notice_edit(params: dict = None, notice_id: int = None, search_params: dict = None):
        """
        提醒修改
        :param params: 要修改的参数
        :param notice_id: 根据ID修改数据
        :param search_params: 根据查询参数，查询单条或者多条数据。两者二选一，不可以同时使用
        :return: data,err
        """
        # 参数为空判断
        notice_id = notice_id or params.pop("notice_id", None)
        if not notice_id and not search_params:
            return None, "找不到可修改的数据"

        # 需要修改的字段过滤
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "notice_type", "notice_title", "notice_content", "link", "files", "status",
                "created_time", "read_time", "expire_time", "thread_id", "from_user_id",
                "to_user_id", "to_role_id",
            ],
        )
        if params is None:
            return {"msg": "数据并未发生修改"}, None

        # 搜索字段
        search_params = format_params_handle(
            param_dict=search_params,
            filter_filed_list=[
                "id", "id_list", "notice_id_list", "notice_type", "notice_type_id", "notice_type_id_list", "thread_id", "thread_id_list",
                "from_user_id", "from_user_id_list", "to_user_id", "to_user_id_list", "to_role_id", "to_role_id_list", "status",
                "created_time_start", "created_time_end" "read_time_start", "read_time_end", "expire_time_start", "expire_time_end"
            ],
            alias_dict={
                "notice_id_list": "id__in",
                "id_list": "id__in",
                "notice_type_id_list": "notice_type_id__in",
                "thread_id_list": "thread_id__in",
                "from_user_id_list": "from_user_id__in",
                "to_user_id_list": "to_user_id__in",
                "to_role_id_list": "to_role_id__in",
                "created_time_start": "created_time__gte",
                "created_time_end": "created_time__lte",
                "read_time_start": "read_time__get",
                "read_time_end": "read_time__lte",
                "expire_time_start": "expire_time__gte",
                "expire_time_end": "expire_time__lte",
            }
        ) if (search_params and isinstance(search_params, dict)) else None

        # 找到要修改的数据
        notice_obj = None
        if notice_id:
            notice_obj = Notice.objects.filter(id=notice_id)
        if search_params:
            notice_obj = Notice.objects.filter(**search_params)

        if not notice_obj:
            return {"msg": "没有找到要修改的数据"}, None

        # 数据修改
        try:
            notice_obj.update(**filter_params)
        except Exception as e:
            return None, "修改异常:" + str(e)

        return None, None

    @staticmethod
    def notice_add(notice_type_value: str = None, notice_params: dict = None, **kwargs):
        """
        添加提醒
        :param notice_type_value: 信息类型Value
        :param notice_params: 其他的系统参数
        :param kwargs: 聚名参数
        :return: data,err
        """
        if notice_params is None:
            notice_params = {}
        if kwargs is None:
            kwargs = {}

        # 这样做的目的是为了,流程自动化调用该服务.
        notice_params.update(kwargs)
        if notice_type_value:
            notice_type_id = NoticeType.objects.filter(value=notice_type_value).values("id").first()
            notice_params.setdefault("notice_type_id", notice_type_id)

        # 参数校验
        if not notice_params.get("to_user_id") and not notice_params.get("to_role_id"):
            return None, "msg:请检查参数;tip:找不到可发送的对象"

        # 需要修改的字段过滤
        filter_params = format_params_handle(
            param_dict=notice_params,
            filter_filed_list=[
                "notice_type", "notice_type_id", "notice_title", "notice_content", "link", "files",
                "created_time", "read_time", "expire_time", "thread_id", "from_user_id",
                "to_user_id", "to_role_id",
            ],
            alias_dict={"notice_type": "notice_type_id"}
        )

        filter_params["status"] = 0
        try:
            Notice.objects.create(**filter_params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None



# encoding: utf-8
"""
@project: djangoModel->test.py
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 测试脚本 say hello
@created_time: 2023/4/17 21:09
"""

from django.core.management.base import BaseCommand


class Command(BaseCommand):
    # 帮助文本, 一般备注命令的用途及如何使用。
    help = "Print Hello World!"

    # 给命令添加一个名为name的参数
    def add_arguments(self, parser):
        parser.add_argument('name')

    # 核心业务逻辑，通过options字典接收name参数值，拼接字符串后输出
    def handle(self, *args, **options):
        msg = 'Hello World ! ' + options['name']
        self.stdout.write(msg)
        # 此时当你再次运行`python manage.py hello_world John`命令时，你将得到如下输出结果：



# encoding: utf-8
"""
@project: djangoModel->translate_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 分页服务测试接口
@created_time: 2022/11/17 13:53
"""
import json

from rest_framework.views import APIView

from ..services.translate_server import TranslateService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class TranslateApis(APIView):
    @request_params_wrapper
    def get(self, *args, request_params=None, **kwargs, ):
        text_list = request_params.get("text", None)
        if text_list is None:
            return util_response(msg="参数错误", err=1000)
        service = TranslateService()
        data, err = service.translate(text_list)
        return util_response(data=data)

    @request_params_wrapper
    def translate_article(self, *args, request_params=None, **kwargs, ):
        article = request_params.get("article", None)
        if article is None:
            return util_response(msg="参数错误", err=1000)
        app = TranslateService()

        # markdown文章拆分成行文本列表
        old_line_arr = app.article_cut_lines(article)

        # 提取行格式，还有需要翻译的文本
        markdown_symbol_list = []
        translate_list = []
        for line in old_line_arr:
            markdown_symbol, translate_text = app.markdown_paragraph_pick(line)
            markdown_symbol_list.append(markdown_symbol)
            translate_list.append(translate_text)

        # 进行翻译
        result, err = app.translate(translate_list)

        # 还原行文本的行格式
        translated_list = [i["text"] for i in result["data"][0]["translated"]]
        deacidizing_list = []
        for i, j in zip(translated_list, markdown_symbol_list):
            deacidizing_list.append(j + i)

        # 链接还原，还原成文本
        deacidizing = ""
        for i in deacidizing_list:
            deacidizing += app.deacidizing_symbol(i) + "\n"

        # 返回数据
        return util_response(data=deacidizing)



# encoding: utf-8
"""
@project: djangoModel->omnipotence_wrapper
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 万能装饰器
@created_time: 2023/6/30 14:27
"""
from .custom_tool import *

class OmnipotenceWrapper:
    """万能装饰器分发类"""

    default_wrappers = [
        {"path": "xj_role.utils.custom_tool", "function": "request_params_wrapper", "must_run": True},
        {"path": "xj_role.utils.user_wrapper", "function": "user_authentication_force_wrapper", "must_run": True},
        {"path": " xj_role.utils.api_interrupter_wrapper", "function": "api_interrupter_wrapper", "must_run": True},
    ]
    custom_run_wrappers = []  # 用户需要走执行的自定以装饰器
    run_wrappers_instance = []  # 先执行默认的装饰器，然后在执行自定义的装饰器

    def __init__(self, run_wrappers=None):
        # 获取默认执行的装饰器
        run_wrappers, err = force_transform_type(variable=run_wrappers, var_type="list")
        if run_wrappers:
            self.run_wrappers = run_wrappers

    def get_wrappers(self):
        """
        获取所有需要执行的装饰器
        :return:
        """
        for wrapper_map in self.default_wrappers:
            wrapper_function, err = dynamic_load_function(import_path=wrapper_map["path"], function_name=wrapper_map["function"])
            if err:
                continue
            self.run_wrappers_instance.append(wrapper_function)
        return self.run_wrappers_instance

    def run_wrapper(self, *args, to_run_wrapper, api_method, **kwargs):
        """
        执行装饰器方法
        :param to_run_wrapper: 需要执行的装饰器
        :param api_method: 系统的API方法
        """

        @to_run_wrapper
        def inner_run_wrapper(*args, **kwargs):
            if callable(api_method):
                return api_method(*args, **kwargs)
            else:
                return api_method

        return inner_run_wrapper(*args, **kwargs)

    def wrapper(self, *args, func_list=[], **kwargs):
        """
        万能Http接口装饰器
        :param func_list: 用户自定义的装饰器
        :param route:  调用路由
        """

        def wrapper(func):
            def route_register_decorator(*args, **kwargs):
                copy_func = func
                for to_run_wrapper_instance in self.get_wrappers():
                    copy_func = self.run_wrapper(*args, api_method=copy_func, to_run_wrapper=to_run_wrapper_instance, **kwargs)
                return copy_func

            return route_register_decorator

        return wrapper


wrapper = OmnipotenceWrapper()
omnipotence_wrapper = wrapper.wrapper



# encoding: utf-8
"""
@project: djangoModel->modules_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 系统模块API
@created_time: 2023/6/5 9:26
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.xj_module_services import XJModuleServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class ModuleApiView(APIView):
    @api_view(["POST"])
    @user_authentication_force_wrapper
    def init_modules(self, *args, **kwargs):
        data, err = XJModuleServices.init_modules()
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["PUT", "POST"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def module_register(self, *args, request_params, **kwargs):
        data, err = XJModuleServices.module_register(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET", "POST"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def module_list(self, *args, request_params, **kwargs):
        data, err = XJModuleServices.module_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->regin_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis:
@created_time: 2022/6/10 16:08
"""
from django.core.paginator import Paginator

from ..models import Region
from ..utils.custom_tool import format_params_handle


class RegionService():

    def cn_parse_location_code(self, region_code):
        # 中国行政编码解析 province{2},city(2),district(2),town(3),village(3)三位
        # level？ province ==>>village : 1 ===>> 5
        if not region_code or not len(str(region_code)) == 12:
            return None
        location = {}
        region_code = str(region_code)
        location['province'] = self.get_region_info(region_code[0:2] + "0000000000")

        city = self.get_region_info(region_code[0:4] + "00000000")
        location['city'] = city if not city == location['province'] else None

        district = self.get_region_info(region_code[0:6] + "000000")
        location['district'] = district if not district == city else None

        town = self.get_region_info(region_code[0:9] + "000")
        location['town'] = town if not town == district else None

        village = self.get_region_info(region_code[0:])
        location['village'] = village if not town == village else None

        return location

    def get_region_info(self, code):
        # 查询对应的名称
        json_data = Region.objects.filter(code=code, is_delete=0).to_json()
        if not json_data:
            return None
        else:
            return json_data

    def region_tree(self, code=None):
        search_list = Region.objects.all().to_json()
        self.p_code_map = {}
        for i in search_list:
            if not self.p_code_map.get(i["p_code"], None):
                self.p_code_map[i["p_code"]] = [i]
            else:
                self.p_code_map[i["p_code"]].append(i)

        if code:
            search_list = Region.objects.filter(code=code).to_json()

        tree_map = []
        for i in search_list:
            if not i["p_code"] == 0 and code is None:
                continue

            i["child"] = self.get_child(i["code"])
            tree_map.append(i)

        return tree_map, None

    def get_child(self, code=None, ):
        # 树形数据遍历
        child_list = self.p_code_map.pop(code, [])
        if not child_list:
            return []

        for i in child_list:
            i["child"] = self.get_child(i["code"])
        return child_list

    @staticmethod
    def list(params=None):
        if params is None:
            params = {}
        page = params.pop('page', 1)
        size = params.pop('size', 20)

        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["id", "code", "p_code", "name", "level"],
            alias_dict={'name': "name__contains", }
        )
        list_set = Region.objects.filter(is_delete=0).filter(**params).values()
        count = list_set.count()
        page_set = Paginator(list_set, size).get_page(page)

        return {'count': int(count), "page": int(page), "size": int(size), "list": list(page_set.object_list)}, None

    @staticmethod
    def add(data):
        res = Region.objects.filter(code=data['code'], is_delete=0)
        if res:
            return None, "该数据已存在"
        instance = Region.objects.create(**data)
        return {"id": instance.id}, None

    @staticmethod
    def edit(data):
        id = data.pop('id', None)
        if not id:
            return None, "ID 必传"
        res_obj = Region.objects.filter(id=id, is_delete=0)
        if not res_obj:
            return None, "数据不存在"
        res_obj.update(**data)
        return None, None

    @staticmethod
    def delete(pk=None):
        if pk is None:
            return None, "参数错误"
        res = Region.objects.filter(id=pk)
        if not res.first():
            return None, None
        res.delete()
        return None, None



# encoding: utf-8
"""
@project: djangoModel->service_register
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 请注意所有模块下面必须填写，模块详情。各模块开发者需注意
@created_time: 2023/1/12 14:29
"""

# 参考案例
short_desc = """
公共服务，非主业务模块。所有函数库保持xj_common模块为最新。
其他模块更系统函数或自定义方法，请同步到该模块utils目录下面。该模块主要提供通用服务功能，供所有模块使用。
"""

module_info = {
    "module_name": "xj_common",
    "module_type": "SUB_MODULE",
    "description": short_desc,
}



from django.contrib import admin

from .models import *


@admin.register(Region)
class RegionAdmin(admin.ModelAdmin):
    list_display = ("id", "code", "p_code", "name", "level", "is_delete", "spell")
    search_fields = ("id", "code", "p_code", "name", "spell")
    fields = ("id", "code", "p_code", "name", "level", "is_delete", "spell")
    readonly_fields = ['id']
    list_per_page = 20


@admin.register(NoticeType)
class NoticeTypeAdmin(admin.ModelAdmin):
    list_display = ("id", "value", "name", "description",)
    search_fields = ("id", "value", "name")
    fields = ("id", "value", "name", "description",)
    readonly_fields = ['id']
    list_per_page = 20


@admin.register(Notice)
class NoticeAdmin(admin.ModelAdmin):
    list_display = (
        "id", "notice_type", "notice_title", "notice_content", "link", "files", "status",
        "created_time", "read_time", "expire_time", "thread_id", "from_user_id", "to_user_id", "to_role_id",
    )
    search_fields = ("id", "notice_type", "from_user_id", "to_user_id", "to_role_id")
    fields = (
        "id", "notice_type", "notice_title", "notice_content", "link", "files", "status",
        "created_time", "read_time", "expire_time", "thread_id", "from_user_id", "to_user_id", "to_role_id"
    )
    readonly_fields = ['id', "created_time"]
    list_per_page = 20


@admin.register(AccessLevel)
class AccessLevelAdmin(admin.ModelAdmin):
    list_display = ("id", "access_code", "access_level", "name", "description")
    search_fields = ("id", "access_code", "access_level",)
    fields = ("id", "access_code", "access_level", "name", "description")
    readonly_fields = ['id']
    list_per_page = 20


@admin.register(Encyclopedia)
class EncyclopediaAdmin(admin.ModelAdmin):
    list_display = ("id", "user_id", "question", "created_time", "updated_time")
    search_fields = ("id", "user_id", "question", "created_time", "updated_time")
    fields = ("id", "user_id", "question", "created_time", "updated_time")
    readonly_fields = ['id']
    list_per_page = 20


@admin.register(AppVersion)
class AppVersionAdmin(admin.ModelAdmin):
    list_display = ("id", "app_system", "version", "version_type", "store_name", "store_url", "is_force_update", "push_time", "created_time")
    search_fields = ("id", "app_system", "version", "version_type", "store_name", "store_url", "is_force_update", "push_time", "created_time")
    fields = ("id", "app_system", "version", "version_type", "store_name", "store_url", "is_force_update", "push_time", "created_time")
    readonly_fields = ['id', "created_time"]
    list_per_page = 20



# encoding: utf-8
"""
@project: djangoModel->access_level_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: APP的管理
@created_time: 2023/4/11 20:54
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.app_version_services import AppVersionServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class AppVersionView(APIView):
    @api_view(["GET"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def app_version_list(self, *args, request_params, **kwargs):
        data, err = AppVersionServices.app_version_list(
            params=request_params,
            need_pagination=request_params.get("need_pagination", True)
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["PUT"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def app_version_edit(self, *args, request_params, **kwargs):
        pk = request_params.get("id") or request_params.get("pk") or kwargs.get("pk")
        data, err = AppVersionServices.app_version_edit(
            pk=pk,
            params=request_params
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["POST"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def app_version_add(self, *args, request_params, **kwargs):
        data, err = AppVersionServices.app_version_add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["DELETE"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def app_version_del(self, *args, request_params, **kwargs):
        pk = request_params.get("id") or request_params.get("pk") or kwargs.get("pk")
        data, err = AppVersionServices.app_version_del(pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET"])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def validate_version(self, *args, request_params, **kwargs):
        data, err = AppVersionServices.validate_version(
            version=request_params.get("version"),
            system=request_params.get("system")
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->encyclopedia_services
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 百科接口，基于chatgpt引擎开放的接口
@created_time: 2023/5/24 22:31
"""

import requests

from config.config import JConfig
from ..models import Encyclopedia
from ..utils.custom_tool import force_transform_type

config = JConfig()
open_api_key = config.get("xj_common", "open_api_key", "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")
headers = {
    'Authorization': f"Bearer {open_api_key}",
}


class EncyclopediaService():
    @staticmethod
    def ask_question(params: dict = None, user_info: dict = None):
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        question = params.get("question")
        if not user_info.get("user_id"):
            return None, "不是一个有效用户ID"

        temperature = params.get("temperature", 0.8)
        temperature, is_pass = force_transform_type(variable=temperature, var_type="float", default=0.8)
        temperature = temperature if 1 >= temperature > 0.2 else 0.8

        if not question:
            return None, "没有收到您的问题"
        json_data = {
            'model': 'gpt-3.5-turbo',
            'messages': [
                {
                    'role': 'user',
                    'content': question,
                },
            ],
            'temperature': temperature,
        }
        proxies_port = config.getint("main", "proxies_port", 10809)
        proxies = {
            'http': 'http://127.0.0.1:' + str(proxies_port),
            'https': 'http://127.0.0.1:' + str(proxies_port)
        }
        response = requests.post('https://api.openai.com/v1/chat/completions', headers=headers, json=json_data, proxies=proxies)
        res_json = response.json()
        encyclopedia_obj = Encyclopedia(
            user_id=user_info.get("user_id"),
            question=question,
            snapshot=res_json.get("choices"),
        )
        encyclopedia_obj.save()
        return res_json, None



# encoding: utf-8
"""
@project: djangoModel->access_level_services
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 访问级别服务
@created_time: 2023/4/11 20:56
"""
from ..models import AccessLevel


class AccessLevelService():
    @staticmethod
    def access_level_list():
        all_access_level = list(AccessLevel.objects.values("id", "access_code", "access_level", "name", "description"))
        return all_access_level, None



# encoding: utf-8
"""
@project: djangoModel->service_register
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 对外开放服务调用注册白名单
@created_time: 2023/1/12 14:29
"""

import xj_common
from .services import notice_services
from .utils.service_manager import ServiceManager

server_manager = ServiceManager()
data, err = server_manager.module_register(xj_common.module_info)


# ============= section 微服务测试放啊 start =========================
def hello(*args, name="word", **kwargs):
    """测试统一主次RPC方法注入"""
    return {"msg": "Hi " + str(name) + "!!!"}


# ============= section 微服务测试放啊 end =========================

# 对外服务白名单
register_list = [
    {
        "service_name": "hello",
        "io_mode": "OTHER",
        "data_type": "OTHER",
        "description": "测试统一注册RPC方法注入",
        "pointer": hello
    },
    {
        "service_name": "notice_add",
        "io_mode": "INSERT",
        "data_type": "MAIN",
        "description": "添加提醒",
        "pointer": notice_services.NoticeServices.notice_add
    },
    {
        "service_name": "notice_edit",
        "io_mode": "UPDATE",
        "data_type": "MAIN",
        "description": "提醒修改",
        "pointer": notice_services.NoticeServices.notice_edit
    }
]


# 遍历注册
def register():
    server_manager.register(register_list, xj_common)


if __name__ == '__main__':
    register()



# encoding: utf-8
"""
@project: djangoModel->encyclopedia_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 百科接口
@created_time: 2023/5/25 8:42
"""
from rest_framework.views import APIView

from ..services.encyclopedia_services import EncyclopediaService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper


class EncyclopediaApis(APIView):

    @user_authentication_force_wrapper
    @request_params_wrapper
    def ask_question(self, *args, request_params=None, user_info, **kwargs):
        if request_params is None:
            request_params = {}
        data, err = EncyclopediaService.ask_question(params=request_params, user_info=user_info)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->notice_services
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: APP的版本管理服务
@created_time: 2023/3/1 11:08
"""

from django.core.paginator import Paginator, EmptyPage

from ..models import AppVersion
from ..utils.custom_tool import format_params_handle, force_transform_type


class AppVersionServices():
    """消息提醒服务类"""

    @staticmethod
    def validate_version(system: str = None, version: str = None):
        system, void = force_transform_type(variable=system, var_type="str")
        version, void = force_transform_type(variable=version, var_type="str")
        if not version or not system:
            return {}, "请检查版本号或者系统类型"

        # 检查当前版本
        app_version_obj = AppVersion.objects.filter(app_system=system, version=version).values(
            "id", "app_system", "version", "version_type", "store_name", "store_url",
            "is_force_update", "push_time", "created_time", "version_detail"
        ).first()
        if not app_version_obj:
            return {}, "请检查版本号或者系统类型"
        # 检查最新版本
        news_version = AppVersion.objects.filter(app_system=system).order_by("-version").values(
            "id", "app_system", "version", "version_type", "store_name", "store_url",
            "is_force_update", "push_time", "created_time", "version_detail"
        ).first()

        if app_version_obj.get("version") == news_version.get("version"):
            app_version_obj["is_news"] = True
            app_version_obj["is_force_update"] = False
        else:
            app_version_obj["is_news"] = False
            app_version_obj["is_force_update"] = news_version.get("is_force_update", False)
            app_version_obj["news_version"] = news_version.get("version")
        return app_version_obj, None

    @staticmethod
    def app_version_list(params: dict = None, need_pagination: int = 0):
        if params is None:
            params = {}
        page = params.pop("page", 1)
        size = params.pop("size", 10)
        sort = params.pop("sort", "version")
        sort = sort if sort in ["version", "-version", "push_time", "-push_time", "created_time", "-created_time"] else "-version"
        # 参数过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id", "app_system", "version", "version_type", "store_name", "store_url",
                "is_force_update|int", "push_time", "min_version", "max_version",
                "push_time_start", "push_time_end"
            ],
            alias_dict={
                "max_version": "version__lte", "min_version": "version__gte",
                "push_time_start": "push_time__gte", "push_time_end": "push_time__lte"
            }
        )
        # 构建ORM
        app_version_obj = AppVersion.objects.filter(**params).order_by(sort).values(
            "id", "app_system", "version", "version_type", "store_name", "store_url",
            "is_force_update", "push_time", "created_time", "version_detail"
        )
        # 不许分页直接返回
        if not need_pagination:
            app_version_list = list(app_version_obj)
            return app_version_list, None

        total = app_version_obj.count()
        paginator = Paginator(app_version_obj, size)
        try:
            app_version_obj = paginator.page(page)
        except EmptyPage:
            app_version_obj = paginator.page(paginator.num_pages)
        except Exception as e:
            return None, f'{str(e)}'

        return {"page": int(page), "size": int(size), "total": total, "list": list(app_version_obj.object_list)}, None

    @staticmethod
    def app_version_edit(pk: int = None, params: dict = None, **kwargs):
        """
        修改系统版本
        :param pk: 修改主键
        :param params: 其他的系统参数
        :param kwargs: 聚名参数
        :return: data,err
        """
        pk, err = force_transform_type(variable=pk, var_type="int")
        if not pk:
            return None, "找不到修改的版本"
        params, void = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, void = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)

        version_obj = AppVersion.objects.filter(id=pk)
        if not version_obj.first():
            return None, "数据不存在"

        # 需要修改的字段过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "app_system|str", "version|str", "version_type|str", "store_name|str", "store_url|str", "is_force_update|int", "push_time|date",
                "version_detail|str"
            ]
        )
        params.setdefault("is_force_update", 0)

        try:
            version_obj.update(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def app_version_add(params: dict = None, **kwargs):
        """
        添加系统版本
        :param params: 其他的系统参数
        :param kwargs: 聚名参数
        :return: data,err
        """
        params, void = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, void = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)

        # 需要修改的字段过滤
        try:
            params = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                filter_filed_list=[
                    "app_system|str", "version|str", "version_type|str", "store_name|str", "store_url|str", "is_force_update|int", "push_time|date", "version_detail|str"
                ]
            )

            # app系统
            must_key = ["app_system", "version"]
            for k in must_key:
                if not params.get(k):
                    return None, k + " 必填"
            params.setdefault("is_force_update", 0)

            # 版本
            if AppVersion.objects.filter(app_system=params.get("app_system"), version=params.get("version")).first():
                return None, "该版本已经发布，请确认版本号和系统类型"

        except ValueError as e:
            return None, str(e)

        try:
            AppVersion.objects.create(**params)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def app_version_del(pk: int = None):
        """
        删除系统版本
        :param pk: 修改主键
        :param params: 其他的系统参数
        :param kwargs: 聚名参数
        :return: data,err
        """
        pk, err = force_transform_type(variable=pk, var_type="int")
        if not pk:
            return None, "找不到修改的版本"

        version_obj = AppVersion.objects.filter(id=pk)
        if not version_obj.first():
            return None, "数据不存在"

        # 需要修改的字段过滤
        try:
            version_obj.delete()
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None



from django.db import models, connection

SHARD_TABLE_NUM = 3


class Region(models.Model):
    id = models.AutoField("ID", primary_key=True)
    code = models.IntegerField('行政编码', null=False, default=0)
    p_code = models.IntegerField('上级行政编码', null=False, default=0)
    name = models.CharField('地区名称', null=False, default="", max_length=50)
    level = models.IntegerField('行政划分等级', null=False, default="0")
    is_delete = models.IntegerField('是否删除', null=False, default=0)
    spell = models.CharField('拼音', max_length=50, null=False, default="")

    class Meta:
        managed = False
        db_table = "append_public_region"
        verbose_name = "行政区编码表"
        verbose_name_plural = verbose_name


class NoticeType(models.Model):
    class Meta:
        managed = False
        db_table = "append_public_notice_type"
        verbose_name = "提醒类型表"
        verbose_name_plural = verbose_name

    id = models.AutoField("ID", primary_key=True)
    value = models.CharField(max_length=25, null=True, blank=True, default="", verbose_name='提醒类型', )
    name = models.CharField(max_length=255, null=True, blank=True, default="", verbose_name='提醒类型名称', )
    description = models.CharField(max_length=255, null=True, blank=True, default="", verbose_name='类型描述', )

    def __str__(self):
        return self.name


class Notice(models.Model):
    class Meta:
        managed = False
        db_table = "append_public_notice"
        verbose_name = "提醒表"
        verbose_name_plural = verbose_name

    status_choice = [(0, '未读'), (1, '已读'), (3, '已删除')]

    id = models.AutoField("ID", primary_key=True)
    notice_type = models.ForeignKey(to=NoticeType, on_delete=models.DO_NOTHING, verbose_name='提醒类型', null=False, blank=False, )
    notice_title = models.CharField(max_length=255, null=False, blank=False, verbose_name='标题')
    notice_content = models.CharField(max_length=1250, null=False, blank=False, verbose_name='内容', default="")
    link = models.CharField(max_length=1500, null=False, blank=False, verbose_name='跳转链接', default="", help_text="跳转链接参数使用")
    files = models.JSONField(null=False, blank=False, verbose_name='文件json', default={}, help_text="消息提醒所带素材")
    status = models.IntegerField(null=False, blank=False, verbose_name='状态', default=0, choices=status_choice, help_text="0：未读；1：已读；2:已删除")
    created_time = models.DateTimeField(verbose_name='发出时间', auto_now_add=True)
    read_time = models.DateTimeField(null=True, blank=True, verbose_name='已读时间')
    expire_time = models.DateTimeField(null=True, blank=True, verbose_name='到期时间')
    thread_id = models.IntegerField(null=True, blank=True, verbose_name='信息（模板）ID', default=0)
    from_user_id = models.IntegerField(null=True, blank=True, verbose_name='发出提醒用户ID', default=0)
    to_user_id = models.IntegerField(null=True, blank=True, verbose_name='提醒用户ID', default=0)
    to_role_id = models.IntegerField(null=True, blank=True, verbose_name='提醒角色ID', default=0)


class AccessLevel(models.Model):
    id = models.AutoField("ID", primary_key=True)
    access_code = models.IntegerField(null=True, blank=True, verbose_name='访问级别的数字编码', default=0, help_text="注意：数值越大，权限越大。高访问级别可看到小于该访问级别的内容")
    access_level = models.CharField(max_length=25, null=True, blank=True, verbose_name='访问级别KEY', )
    name = models.CharField(max_length=255, null=False, blank=True, verbose_name='访问级别名称', default="", )
    description = models.CharField(max_length=255, null=True, blank=True, verbose_name='相关描述', default="", )

    class Meta:
        managed = False
        unique_together = ["access_level", "access_code"]
        db_table = "append_access_level"
        verbose_name = "访问级别表"
        verbose_name_plural = verbose_name


class Encyclopedia(models.Model):
    class Meta:
        managed = False
        db_table = "append_public_ecyclopedia"
        verbose_name = "百科问答表"
        verbose_name_plural = verbose_name

    id = models.AutoField("ID", primary_key=True)
    user_id = models.IntegerField(null=True, blank=True, verbose_name='用户ID')
    question = models.TextField(null=True, blank=True, verbose_name='问题描述')
    snapshot = models.JSONField(null=True, blank=True, verbose_name='快照')
    created_time = models.DateTimeField(auto_now_add=True, null=True, blank=True)
    updated_time = models.DateTimeField(auto_now=True, null=True, blank=True)


class ModuleServices(models.Model):
    class Meta:
        managed = False
        db_table = "append_module_services"
        verbose_name = "模型服务映射表"
        verbose_name_plural = verbose_name

    @classmethod
    def truncate(cls):
        with connection.cursor() as cursor:
            cursor.execute('TRUNCATE `{}`'.format(cls._meta.db_table))

    io_mode_choices = (
        ("SELECT", 'SELECT'),
        ("INSERT", 'INSERT'),
        ("DELETE", 'DELETE'),
        ("UPDATE", 'UPDATE'),
        ("NO_IO", "NO_IO")
    )
    data_type_choices = (
        ("MAIN", 'MAIN'),
        ("RECORD", 'RECORD'),
        ("DETAIL", 'DETAIL'),
        ("NO_INPUT", "NO_INPUT")
    )
    id = models.AutoField("ID", primary_key=True)
    module_name = models.CharField(null=True, max_length=120, blank=True, verbose_name='模型名称')
    service_name = models.CharField(null=True, max_length=120, blank=True, verbose_name='服务名称')
    description = models.CharField(null=True, max_length=5000, blank=True, verbose_name='描述')
    io_mode = models.CharField(choices=io_mode_choices, max_length=50, null=True, blank=True, verbose_name="IO模型")
    data_type = models.CharField(choices=data_type_choices, max_length=50, null=True, blank=True, verbose_name="数据类型")
    created_time = models.DateTimeField(auto_now_add=True, null=True, blank=True)


class Modules(models.Model):
    class Meta:
        managed = False
        db_table = "append_modules"
        verbose_name = "模型模块注册表"
        verbose_name_plural = verbose_name

    @classmethod
    def truncate(cls):
        with connection.cursor() as cursor:
            cursor.execute('TRUNCATE `{}`'.format(cls._meta.db_table))

    module_type_choices = (
        ("SYSTEM", 'SYSTEM'),
        ("MAIN_MODULE", 'MAIN_MODULE'),
        ("SUB_MODULE", 'SUB_MODULE')
    )
    id = models.AutoField("ID", primary_key=True)
    module_name = models.CharField(null=True, max_length=120, blank=True, verbose_name='模型名称')
    module_type = models.CharField(choices=module_type_choices, max_length=50, null=True, blank=True, verbose_name="数据类型")
    description = models.CharField(null=True, max_length=5000, blank=True, verbose_name='描述')
    created_time = models.DateTimeField(null=True, blank=True, auto_now_add=True)


class AppVersion(models.Model):
    class Meta:
        managed = False
        db_table = "append_app_version"
        verbose_name = "APP系统表"
        verbose_name_plural = verbose_name

    app_system_choices = (
        ("ios", 'ios'),
        ("android", 'android')
    )
    version_type_choices = (
        ("alpha", 'Alpha'),
        ("beta", 'Beta'),
        ("rc", 'RC'),
        ("release", 'Release')
    )

    id = models.AutoField("ID", primary_key=True)
    app_system = models.CharField(null=True, max_length=50, choices=app_system_choices, blank=True, verbose_name='app系统')
    version = models.CharField(max_length=255, null=True, blank=True, verbose_name="版本")
    version_type = models.CharField(null=True, max_length=255, choices=version_type_choices, blank=True, verbose_name='是否强制更新')
    store_name = models.CharField(null=True, max_length=255, blank=True, verbose_name='渠道名')
    store_url = models.CharField(null=True, max_length=5000, blank=True, verbose_name='渠道商跳转路由')
    version_detail = models.CharField(null=True, max_length=10000, blank=True, verbose_name='更新内容')
    is_force_update = models.IntegerField(null=True, default=0, blank=True, verbose_name='是否强制更新')
    push_time = models.DateTimeField(null=True, blank=True, auto_now_add=True, verbose_name='推送时间')
    created_time = models.DateTimeField(null=True, blank=True, auto_now_add=True, verbose_name='创建时间')



from django.urls import re_path

from .apis import notice_apis, region_api, translate_apis, access_level_apis, service_manage_api, encyclopedia_api, modules_apis, module_service_apis, app_versionl_apis
from .service_register import register

register()
urlpatterns = [
    # 地区相关API
    re_path(r'^region_list/?$', region_api.RegionAPIS.region_list, ),
    re_path(r'^region_tree/?$', region_api.RegionAPIS.get_region_tree, ),
    re_path(r'^region_add/?$', region_api.RegionAPIS.region_add, ),
    re_path(r'^region_edit/?$', region_api.RegionAPIS.region_edit, ),
    re_path(r'^region_del/?$', region_api.RegionAPIS.region_del, ),

    # 公共翻译
    re_path(r'^translate_article/?$', translate_apis.TranslateApis.translate_article, ),
    re_path(r'^translate_test/?$', translate_apis.TranslateApis.as_view(), ),

    # 提醒相关接口
    re_path(r'^notice_add/?$', notice_apis.NoticeApis.notice_add),
    re_path(r'^notice_edit/?$', notice_apis.NoticeApis.notice_edit),
    re_path(r'^notice_list/?$', notice_apis.NoticeApis.notice_list),
    re_path(r'^notice_type_list/?$', notice_apis.NoticeApis.type_list),

    # 访问级别
    re_path(r'^access_level_list/?$', access_level_apis.AccessLevelAPIView.access_level_list),

    # 服务治理
    re_path(r'^find_service_test/?$', service_manage_api.ServiceManageApis.find_service_test),  # 发现服务

    # chatGPT接口
    re_path(r'^ask_question/?$', encyclopedia_api.EncyclopediaApis.ask_question),

    # 模块与开放服务列表
    re_path(r'^module_list/?$', modules_apis.ModuleApiView.module_list),
    re_path(r'^module_register/?$', modules_apis.ModuleApiView.module_register),
    re_path(r'^services_list/?$', module_service_apis.ModuleApiView.services_list),

    # app版本管理接口
    re_path(r'^app_version_list/?$', app_versionl_apis.AppVersionView.app_version_list, name="系统版本列表"),
    re_path(r'^app_version_edit/?(?P<pk>\d+)?$', app_versionl_apis.AppVersionView.app_version_edit, name="系统版本编辑"),
    re_path(r'^app_version_add/?(?P<pk>\d+)?$', app_versionl_apis.AppVersionView.app_version_add, name="系统版本添加"),
    re_path(r'^app_version_del/?(?P<pk>\d+)?$', app_versionl_apis.AppVersionView.app_version_del, name="系统版本删除"),
    re_path(r'^validate_version/?$', app_versionl_apis.AppVersionView.validate_version, name="系统版本删除")

]



# encoding: utf-8
"""
@project: djangoModel->access_level_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 访问级别列表
@created_time: 2023/4/11 20:54
"""
from rest_framework.views import APIView

from ..services.access_level_services import AccessLevelService
from ..utils.custom_response import util_response


class AccessLevelAPIView(APIView):
    def access_level_list(self, *args, **kwargs):
        access_level_list, err = AccessLevelService.access_level_list()
        if err:
            return util_response(msg=err)
        return util_response(data=access_level_list)



# encoding: utf-8
"""
@project: djangoModel->service_manager
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 服务治理者
@created_time: 2023/5/7 15:43
"""
import importlib
import sys

import consul
import jsonrpc
from jsonrpc.proxy import ServiceProxy
import requests

from .custom_tool import force_transform_type, singleton_decorator


@singleton_decorator
class ServiceManager():
    """服务管理者"""
    providers = None  # 服务提供者
    service_clock_key = "SERVICE_CLOCK"  # 倒计时

    def __init__(self, host="127.0.0.1", port=8500):
        port, is_pass = force_transform_type(variable=port, var_type="int")
        self.consul_client = consul.Consul(host=host, port=port)

    def find_providers(self, service_id=None):
        """
        服务治理，再consul服务中发现服务提供者
        :return:
        """
        try:
            self.providers = self.consul_client.agent.services()
            if service_id:
                self.providers = self.providers.get(str(service_id), None)
            # # 创建定时锁，到期会从新拉取服务
            # has_clock = self.redis_client.get(self.service_clock_key)
            # if not has_clock:
            #     self.redis_client.set(self.service_clock_key, "")
            #     self.redis_client.expire(self.service_clock_key, 10)
            return self.providers, None
        except requests.exceptions.ConnectionError:
            return None, "无法连接服务治理服务端"

    def find_rpc_service(self, service_id=None, arg_version="2.0", system_rpc_entry="rpc", **kwargs):
        """
        服务之间通过rpc调用
        :param arg_version: 传递路由参数类型 @note 1.0：arg；2.0 kwargs;
        :param service_id: 服务ID
        :param system_rpc_entry: 服务监听基础路由
        :return: service_cal_handle, err
        """
        # ============== section 再consul获取活跃健康可调用的服务器 start =====================
        if not service_id:
            return None, "service_id无效，没有可调用服务"

        providers, err = self.find_providers()
        if not providers is None:
            provider = providers.get(str(service_id), {})
        else:
            return None, "没有可调用服务。请检查consul是否开启，并检查服务提供则是否运行。"
        # ============== section 再consul获取活跃健康可调用的服务器 end    =====================

        # ============== section 得到获取的服务名单,并开始调用服务 start    =====================
        tagged_addresses = provider.get("TaggedAddresses", {}).get("lan_ipv4", {}).get("Address", "127.0.0.1")
        port = provider.get("TaggedAddresses", {}).get("lan_ipv4", {}).get("Port", "8000")
        request_path = "http://" + tagged_addresses + ":" + str(port) + "/" + system_rpc_entry
        try:
            service = ServiceProxy(request_path, version=arg_version)
        except jsonrpc.proxy.ServiceProxyException:
            return None, "连接服务超时"
        return service, None
        # ============== section 得到获取的服务名单,并开始调用服务 end    =====================

    def put_rpc_service(self, route="", method=None, **kwargs):
        """
        注册方法提供rpc调用
        :param route: 路由
        :param method: 方法指针
        :return: None
        """

        @jsonrpc.jsonrpc_method(route)
        def rpc_register(*args, **kwargs):
            return method(*args, **kwargs)

        return rpc_register

    def dynamic_load_class(self, import_path: str = None, class_name: str = None, find_service_in_consul=False, **kwargs):
        """
        动态加载模块中的类,返回类的指针
        :param find_service_in_consul: 是否再consul中发现服务
        :param import_path: 导入类的文件路径
        :param class_name: 导入文件中的哪一个类
        :return: class_instance,err_msg
        """
        if find_service_in_consul:
            return self.find_providers(**kwargs)
        try:
            class_instance = getattr(sys.modules.get(import_path), class_name, None)
            if class_instance is None:
                from xj_common.services.xj_module_services import XJModuleServices
                import_module = importlib.import_module(import_path)
                class_instance = getattr(import_module, class_name)
            return class_instance, None
        except AttributeError as e:
            return None, "系统中不存在该模块"
        except Exception as e:
            return None, str(e)

    def module_register(self, module_info):
        """模块统一注册入口"""
        XJModuleServices, import_err = self.dynamic_load_class(
            import_path="xj_common.services.xj_module_services",
            class_name="XJModuleServices"
        )
        if import_err:
            return None, "请安装公共模块"
        return XJModuleServices.module_register(module_info)

    def service_register(self, params, **kwargs):
        """模块统一注册入口"""
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)
        XJModuleServiceServices, import_err = self.dynamic_load_class(
            import_path="xj_common.services.xj_module_service_services",
            class_name="XJModuleServiceServices"
        )
        if import_err:
            return None, "请安装公共模块"
        return XJModuleServiceServices.services_register(params)

    def register(self, register_list: list = None, module_pointer=None):
        """服务注册统一入口"""
        for i in register_list:
            setattr(module_pointer, i["service_name"], i["pointer"])
            self.put_rpc_service(route=i["service_name"], method=i["pointer"])
            self.service_register(i, module_name=module_pointer.__name__)



# encoding: utf-8
"""
@project: djangoModel->main
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 任务调度器
@created_time: 2022/7/26 13:28
"""
import os
import sys
import time

import schedule

sys.path.append("D:\\PySet\\djangoModel")
os.environ.setdefault('DJANGO_SETTINGS_MODULE', "main.settings")
import django

django.setup()

from command.bxtx_enroll_timer import ClockService

clocker = ClockService()
# 任务注册
schedule.every(2).seconds.do(clocker.check_end_clock)

while True:
    # 定时任务守护程序
    schedule.run_pending()
    time.sleep(1)



# encoding: utf-8
"""
@project: djangoModel->service_manage_api
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 服务治理API
@created_time: 2023/5/7 15:32
"""
from rest_framework.views import APIView

from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.service_manager import ServiceManager


class ServiceManageApis(APIView):
    @request_params_wrapper
    def find_service_test(self, *args, request_params, **kwargs):
        service_id = request_params.get("service_id", 5521)
        name = request_params.get("service_id", "sunkaiyan")
        service_manager = ServiceManager()
        service, err = service_manager.find_service(service_id=service_id)
        if err:
            return util_response(msg=err)
        return util_response(data=service.hello(name=name))



# encoding: utf-8
"""
@project: djangoModel->open_service_seervices
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 系统服务
@created_time: 2023/6/5 9:30
"""
from django.core.paginator import Paginator, EmptyPage

from ..models import ModuleServices
from ..utils.custom_tool import force_transform_type, format_params_handle


class XJModuleServiceServices(object):
    @staticmethod
    def init_services():
        """初始化数据表"""
        try:
            ModuleServices.truncate()
        except Exception as e:
            return None, str(e)
        return None, None

    @staticmethod
    def services_register(params: str = None, **kwargs):
        """
        模块注册
        :param params: 参数
        :param kwargs: 聚名参数
        :return: data,err
        """
        params, is_pass = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)

        # 需要修改的字段过滤
        filter_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["module_name", "service_name", "description", "io_mode", "data_type"],
        )
        must_keys = ["module_name", "service_name", "io_mode", "data_type"]
        for i in must_keys:
            if not filter_params.get(i):
                return None, str(i) + "不可以为空"

        # 构建ORM
        try:
            orm_obj = ModuleServices(**filter_params)
            orm_obj.save()
        except Exception as e:
            return None, "修改异常:" + str(e)
        return None, None

    @staticmethod
    def services_list(params: dict = None, need_pagination: int = 0, **kwargs):
        """
        模块列表
        :param params:
        :param need_pagination:
        :param kwargs:
        :return:
        """
        params, is_pass = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)

        page, is_pass = force_transform_type(variable=params.pop("page", 1), var_type="int", default=1)
        size, is_pass = force_transform_type(variable=params.pop("size", 10), var_type="int", default=10)
        if size > 200:
            return None, "分页不可以超过200"

        # 参数过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=["module_name", "service_name", "io_mode", "data_type"]
        )
        services_obj = ModuleServices.objects.extra(select={
            'created_time': 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'
        }).filter(**params).values(
            "id", "module_name", "service_name", "description", "io_mode", "data_type", "created_time"
        )
        total = services_obj.count()

        # 不许分页直接返回
        if not need_pagination and total <= 200:
            notice_type_list = list(services_obj)
            return notice_type_list, None

        paginator = Paginator(services_obj, size)
        try:
            services_obj = paginator.page(page)
        except EmptyPage:
            return {"page": int(page), "size": int(size), "total": total, "list": []}
        except Exception as e:
            return None, f'{str(e)}'

        return {"page": int(page), "size": int(size), "total": total, "list": list(services_obj.object_list)}, None
